/******************************************************************/
/* filename: cf_pe.c */
/* version : 1.00  */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
#include "cf_pe.h"
#include "processConfig.h"
#include <stdlib.h>
#include <signal.h>
#include "debug.h"

/******************************************************************/
/* External declaration of [defined in cp.c] */
extern int CSAP_datareq_pipe[2];
extern int CSAP_dataind_pipe[2];
extern int exit_cf_st_pipe[2]; /* We use only the write pipe [1]! */
extern int exit_cf_rt_pipe[2]; /* We use only the write pipe [1]! */

/******************************************************************/
/* External declaration of processes */
extern int cf_st(); /* See cf_st.c */
extern int cf_rt(); /* See cf_rt.c */

/******************************************************************/
/* function prototying (forward declaration) */
int cf_pe(struct ConfPartnersStruc *pp);

/******************************************************************/
/* Local CF_PE pipe declaration */
int cf_st2cf_rt_pipe[2];
int cf_rt2cf_st_pipe[2];

/******************************************************************/
/* Declaration of local variables */
static int PId=0;				/* process identifier */ 

/******************************************************************/
 static void terminate(int unused)
 {
  if (PId > 0) 
  {
   kill(PId, SIGTERM);
   Debug(Dbg,"cf_pe.c: terminated %d\n", PId);
   PId = 0;
  }
  exit(0);
 }
/******************************************************************/

int cf_pe(struct ConfPartnersStruc *pp)
{
 /* Initialize conference variable */
 conference.active = 0;

 /* cf_pe_init means creation of the CF_PE pipes for the cf_st() and cf_rt() */
 if (pipe(cf_st2cf_rt_pipe)!=0)
 {
  Error(Cf_pe,"cf_pe.c: Creating cf_st2cf_rt_pipe\n");
  return(-1);
 }
 if (pipe(cf_rt2cf_st_pipe)!=0)
 {
  Error(Cf_pe,"cf_pe.c: Creating cf_rt2cf_st_pipe\n");
  /* Note that cf_rt2cf_st_pipe are not closed! */
  close( cf_st2cf_rt_pipe[0]);
  close( cf_st2cf_rt_pipe[1]);
  return (-1);
 }
 
 /* Set desired termination signals */
 signal(SIGHUP, terminate);
 signal(SIGTERM, terminate);
 signal(SIGQUIT, terminate);
 signal(SIGKILL, terminate);
 signal(SIGINT, terminate);
 signal(SIGCHLD, terminate); 

 if ((PId = fork()) == -1)
 {
  Error(Cf_pe,"cf_pe.c: Executing fork()\n");
  return(-1);
 }

 if (PId)
 {
  int rvalue;

  /* close the pipes which the cf_st() does not use! */
  close(CSAP_dataind_pipe[1]);
  close(cf_rt2cf_st_pipe[1]);
  close(cf_st2cf_rt_pipe[0]);

  /* Startup the cf_st() */
  rvalue = cf_st(pp);
  terminate(0);
  Debug(Dbg,"cf_pe.c: Exit Conference Send Task detected...");

  /* close pipes */
  close(CSAP_datareq_pipe[0]);
  close(cf_rt2cf_st_pipe[0]);
  close(cf_st2cf_rt_pipe[1]);
  
  return rvalue;
 }
 else
 {
  int rvalue;

  /* close the pipes which the cf_rt() process does not use! */
  close(CSAP_datareq_pipe[0]);
  close(cf_rt2cf_st_pipe[0]);
  close(cf_st2cf_rt_pipe[1]);

  /* startup cf_rt() */
  rvalue = cf_rt(pp);
  terminate(0);
  Debug(Dbg,"cf_pe.c: Exit Conference Receive Task detected...");
  
  /* close pipe */
  close(CSAP_dataind_pipe[1]);
  close(cf_rt2cf_st_pipe[1]);
  close(cf_st2cf_rt_pipe[0]);

  return rvalue;
 }
}

