/******************************************************************/
/* filename: cf_st.c */
/* version : 1.00 */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
#include "cf_pe.h"
#include "processConfig.h"
#include "debug.h"

/******************************************************************/
/* external declaration of pipes */
extern int CSAP_datareq_pipe[2];
extern int cf_st2cf_rt_pipe[2];
extern int cf_rt2cf_st_pipe[2];
extern int MSAP_datareq_pipe[2];

/******************************************************************/
/* function prototying (forward declaration) */
int cf_st(struct ConfPartnersStruc *pp);
int InsertPCPartners(struct datareq_mdu_Type *mdu, 
 struct ConfPartnersStruc *pp);
int InsertPartners(struct datareq_mdu_Type *mdu, 
 struct partner_Type pp[MAXPARTNERS+1]);
int InsertConfid(char id1[CONFIDLENGTH], char id2[CONFIDLENGTH]);
int InsertUsertitle(char id1[USERTITLELENGTH], char id2[USERTITLELENGTH]);
int ShowPartners(int level, struct partner_Type partner[MAXPARTNERS+1]);
int UpdateConference(struct conference_Type *conference,
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH], int active);
int new_join_pdu(struct join_pdu_Type *pdu,
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH]);
int new_leave_pdu(struct leave_pdu_Type *pdu,
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH]);
int new_answer_pdu(struct answer_pdu_Type *pdu, 
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH]);
int new_data_pdu(struct data_pdu_Type *pdu, char length, 
 char data[DATALENGTH]);
int new_rtst_pdu(struct rtst_pdu_Type *rpdu, char type, int srcAddr, 
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH]);

/******************************************************************/
/* external declaration of variables */
extern struct conference_Type conference;

int cf_st(struct ConfPartnersStruc *pp)
{  
 struct cdu_Type cdu;
 fd_set read_mask;
 Debug(Cf_st,"cf_st.c: Entering the Sending Task Loop!");
 while(1)  
 {  
  FD_ZERO(&read_mask);  
  FD_SET(CSAP_datareq_pipe[0], &read_mask);  
  FD_SET(cf_rt2cf_st_pipe[0], &read_mask); 

  select(FD_SETSIZE, &read_mask, (fd_set *) 0, (fd_set *) 0, (struct timeval *) 0);

  if (FD_ISSET(cf_rt2cf_st_pipe[0], &read_mask)) 
  {
   struct rtst_pdu_Type rpdu;
   int read_result;
   read_result = read(cf_rt2cf_st_pipe[0], (char *)&rpdu, \
           sizeof(struct rtst_pdu_Type));
   if (read_result < 0)
   {
    Error(Cf_st,"cf_st.c: []  reading from cf_rt2cf_st_pipe[0]!\n");
    return(-1);
   }
   else if (read_result == 0) 
   {
    return(0); /* EOF */
   }
   
   switch (rpdu.type) 
   {

    case JOIN:
	{
     struct datareq_mdu_Type mdu;
	 struct join_pdu_Type *join_pdu;
	 
     Debug(Cf_st,"cf_st.c: Entering the 'rtst' join case!");
 	 Debug(Cf_st,"cf_st.c: join: Insert partner!");
    /* The receive task asks me to send an join-pdu to specific partner */
     mdu.dstAddress[0] = rpdu.dstAddress;
     mdu.dstAddress[1] = 0;

 	 Debug(Cf_st,"cf_st.c: Construct join-pdu!");
     join_pdu = (struct join_pdu_Type *)&mdu.data;
	 new_join_pdu(&(*join_pdu), conference.usertitle, conference.confid);
	 
 	 Debug(Cf_st,"cf_st.c: Send join-pdu!");
 	 if (!mutant(100)) 
 	 {
       write(MSAP_datareq_pipe[1], (char *)&mdu, \
	     sizeof(struct datareq_mdu_Type));
	 }
    break;
    }
	case ANSWER:
    {
	 struct datareq_mdu_Type mdu;
	 struct answer_pdu_Type *answer_pdu;

     Debug(Cf_st,"cf_st.c: Entering the 'rtst' answer case!");
	 /* Insert new partner! */
	 if (!IsConfPartner(conference.partner, rpdu.dstAddress))
	 {
      Debug(Cf_st,"cf_st.c: answer: Add 'rtst' user to partner!");
 	  if (!mutant(348)) 
  	  {
 	   AddNewPartner(&conference.partner, rpdu.dstAddress, rpdu.usertitle);
	  }
	 }

 	 Debug(Cf_st,"cf_st.c: answer: Insert partner!");
     /* The receive task asks me to send an answer-pdu to specific partner */
     mdu.dstAddress[0] = rpdu.dstAddress;
	 mdu.dstAddress[1] = 0;

 	 Debug(Cf_st,"cf_st.c: Construct answer pdu!");
	 answer_pdu = (struct answer_pdu_Type *)&mdu.data;
	 new_answer_pdu(&(*answer_pdu),conference.usertitle, conference.confid);

 	 Debug(Cf_st,"cf_st.c: Send answer-pdu!");
 	 if (!mutant(111)) 
 	 {
      write(MSAP_datareq_pipe[1], (char *)&mdu, \
	    sizeof(struct datareq_mdu_Type));
	 }
     break;
    }
    case DATA:
    {
     Debug(Cf_st,"cf_st.c: Entering the 'rtst' data case!");
     break;
    }
    case LEAVE:
    {

     Debug(Cf_st,"cf_st.c: Entering the 'rtst' leave case!");
 	 Debug(Cf_st,"cf_st.c: Removing participant from partner!");
 	 if (!mutant(214)) 
 	 {
	  RemoveKnownConfPartner(&conference.partner, rpdu.dstAddress);
	 }
     break;
    }
	case NEW:
	{
     Debug(Cf_st,"cf_st.c: Entering the 'rtst' new case!");
 	 Debug(Cf_st,"cf_st.c: Adding participant to partner!");
 	 if (!mutant(462)) 
 	 {
      AddNewPartner(&conference.partner, rpdu.dstAddress, rpdu.usertitle);
	 }
	 break;
	}
    default:
    {
	 Debug(Cf_st,"cf_st.c: rtst: Unknown pdu read from cf_rt2cf_st_pipe!");
     break;
    }
   }
  }

  if (FD_ISSET(CSAP_datareq_pipe[0], &read_mask))  
  { 
   int read_result;
   read_result = read(CSAP_datareq_pipe[0], (char *)&cdu, \
	  sizeof(struct cdu_Type)); 
   if(read_result < 0)  
   {  
	Error(Cf_st,"cf_st.c: Reading from CSAP_datareq_pipe[0]!");
    return(-1);  
   }  
   else if (read_result == 0) 
   {
    return(0); /* EOF */
   }

   switch(cdu.type)  
   {  
    case JOIN:  
    {
	 struct join_cdu_Type *pcdu;
     struct join_pdu_Type *ppdu;
     struct datareq_mdu_Type mdu;

     Debug(Cf_st,"cf_st.c: Entering the CSAP join case!");
	 /* The user wants to join a named conference! */
	 if (conference.active!=0) {
	  Debug(Cf_st,"cf_st.c: JOIN: Different conference is active!");
	  Debug(Cf_st,"cf_st.c: Current status of Conference (at Send Task Side):"); 
	  Debug(Cf_st,"cf_st.c: Conference id :"); 
	  writeTitle(conference.confid);
	  Debug(Cf_st,"cf_st.c: UserTitle :");
	  writeTitle(conference.usertitle);
	  ShowPartners(Details,conference.partner);
	  break;
	 }

     Debug(Cf_st,"cf_st.c: Active conference!");
	 pcdu = (struct join_cdu_Type *)&cdu;

	 /* Inform the user (rt_st()) and all partners that we join a conference */

     Debug(Cf_st,"cf_st.c: Construct join-pdu!");
	 /* First construct the join-pdu! */
	 ppdu = (struct join_pdu_Type *)&mdu.data;
     new_join_pdu(ppdu, pcdu->usertitle, pcdu->confid);

     Debug(Cf_st,"cf_st.c: Sending join-pdu to cf_rt!");
	 /* Inform the reveicing task to do some administration */
	 write(cf_st2cf_rt_pipe[1], (char *)ppdu, sizeof(struct join_pdu_Type));

     Debug(Cf_st,"cf_st.c: Adding joining sender!");
	 /* Do some sending task administration */
 	 if (!mutant(749)) 
 	 {
      UpdateConference(&conference, pcdu->usertitle, pcdu->confid, 1);
	 }

     Debug(Cf_st,"cf_st.c: Construct broadcast mdu!");
	 /* Insert all potential partners (Broadcast) */
	 InsertPCPartners(&mdu, pp);

     Debug(Cf_st,"cf_st.c: Sending broadcast to MSAP!");
 	 if (!mutant(674)) 
 	 {
	  write(MSAP_datareq_pipe[1], (char *)&mdu,sizeof(struct datareq_mdu_Type));
	 }
     break;  
    }  
	case DATAREQ:
	{
	 struct data_pdu_Type *ppdu;
	 struct datareq_cdu_Type *pcdu;
	 struct datareq_mdu_Type mdu;

     Debug(Cf_st,"cf_st.c: Entering the CSAP datareq case!");
     /* The user wants to send a 'message' to all active partners */
	 if (conference.active==0) {
	  Debug(Cf_st,"cf_st.c: DATAREQ: No conference!");
	  break;
	 }

	 Debug(Cf_st,"cf_st.c: Construct data pdu");
	 /* Create an data pdu which contains the string message */
	 pcdu = (struct datareq_cdu_Type *)&cdu;
	 ppdu = (struct data_pdu_Type *)&mdu.data;
     new_data_pdu(&(*ppdu), pcdu->length, pcdu->data);
     
	 Debug(Cf_st,"cf_st.c: Inserting all partners");
	 /* Insert active partners ... */
     InsertPartners(&mdu, conference.partner);

	 Debug(Cf_st,"cf_st.c: Sending data-pdu to all partners.");
	 /* Send mdu packet which contains the data-pdu to active partners... */
 	 if (!mutant(548)) 
 	 {
      write(MSAP_datareq_pipe[1], (char *)&mdu, \
	   sizeof(struct datareq_mdu_Type));
	 }
	 break;
    }
    case DATAIND:  
    {  
     Debug(Cf_st,"cf_st.c: Entering the CSAP dataind case!");
     break;  
    }  
    case LEAVE:  
    {  
	 struct leave_pdu_Type *ppdu;
	 struct datareq_mdu_Type mdu;

     Debug(Cf_st,"cf_st.c: Entering the CSAP leave case!");
	 if (conference.active==0) 
	 {
	  Debug(Cf_st,"cf_st.c: LEAVE: No conference!");
	  break;
	 }

	 Debug(Cf_st,"cf_st.c: Construct leave-pdu!");
	 /* Construct the leave-pdu... */
	 ppdu = (struct leave_pdu_Type *)&mdu.data;
     new_leave_pdu(&(*ppdu), conference.usertitle, conference.confid);

     Debug(Cf_st,"cf_st.c: Sending leave-pdu to cf_rt!");
	 /* Inform the receiving task that the user leave the conference! */
 	 if (!mutant(945)) 
 	 {
	  write(cf_st2cf_rt_pipe[1], (char *)ppdu, sizeof(struct leave_pdu_Type));
	 }

     Debug(Cf_st,"cf_st.c: Clearing conference!");
	 /* Do some local administration! */
 	 if (!mutant(345)) 
 	 {
      UpdateConference(&conference, conference.usertitle, \
	      conference.confid, 0);
	 }
	      
	 Debug(Cf_st,"cf_st.c: Inserting all partners");
	 /* Insert all active partners that this user leaves the conference */
	 InsertPartners(&mdu, conference.partner);

	 Debug(Cf_st,"cf_st.c: Clearing partners!");
	 /* clear partner list? */
 	 if (!mutant(332)) 
 	 {
	  conference.partner[0].msapid = 0;
	 }
	 
	 Debug(Cf_st,"cf_st.c: Sending leave-pdu to all partners.");
 	 if (!mutant(687)) 
 	 {
      write(MSAP_datareq_pipe[1], (char *)&mdu, \
	   sizeof(struct datareq_mdu_Type));
	 }
     break;  
	}
    default:  
    {  
     Debug(Cf_st,"cf_st.c: Entering the CSAP default case: !");
	 break;
    }  
   }
  }
 }
} 
/******************************************************************/

int InsertPCPartners(struct datareq_mdu_Type *mdu, \
  struct ConfPartnersStruc *pp)
{
 int i;
 for (i=0;i<pp->size;i++)
  mdu->dstAddress[i] = pp->Partner[i].msapid;
 mdu->dstAddress[i] = 0; /* closing zero */
 return(0);
}
/******************************************************************/

int InsertPartners(struct datareq_mdu_Type *mdu, \
  struct partner_Type pp[MAXPARTNERS+1])
{
 int i;
 for (i=0; pp[i].msapid!=0; i++)
  mdu->dstAddress[i]=pp[i].msapid;
 mdu->dstAddress[i]=0;
 return(0);
}
/******************************************************************/

int InsertConfid(char id1[CONFIDLENGTH], char id2[CONFIDLENGTH])
{
 int i;
 for (i=0; i<CONFIDLENGTH; i++)
  id2[i] = id1[i];
 return(0);
}
/******************************************************************/
  
int InsertUsertitle(char id1[USERTITLELENGTH], char id2[USERTITLELENGTH])
{
 int i;
 for (i=0; i<USERTITLELENGTH; i++)
  id2[i] = id1[i];
 return(0);
}

/******************************************************************/

int UpdateConference(struct conference_Type *conference,
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH], int active)
{
 InsertUsertitle(usertitle, &(*conference->usertitle));
 InsertConfid(confid, &(*conference->confid));
 conference->active = active;
 return(0);
}
/******************************************************************/

int new_join_pdu(struct join_pdu_Type *pdu,
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH])
{
 pdu->type = JOIN;
 InsertUsertitle(usertitle, &(*pdu->usertitle));
 InsertConfid(confid, &(*pdu->confid));
 return(0);
}

/******************************************************************/

int new_leave_pdu(struct leave_pdu_Type *pdu,
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH])
{
 pdu->type = LEAVE;
 InsertUsertitle(usertitle, &(*pdu->usertitle));
 InsertConfid(confid, &(*pdu->confid));
 return(0);
}
/******************************************************************/

int new_answer_pdu(struct answer_pdu_Type *pdu, 
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH])
{
 pdu->type = ANSWER;
 InsertUsertitle(usertitle, &(*pdu->usertitle));
 InsertConfid(confid, &(*pdu->confid));
 return(0);
}
/******************************************************************/

int new_data_pdu(struct data_pdu_Type *pdu, char length, char data[DATALENGTH])
{
 pdu->type = DATA;
 pdu->length = length;
 memcpy((char *)&pdu->data, (char *)data, (size_t)length);
}
/******************************************************************/

int new_rtst_pdu(struct rtst_pdu_Type *rpdu, char type, int srcAddr, 
 char usertitle[USERTITLELENGTH], char confid[CONFIDLENGTH])
{
 rpdu->type = type;
 rpdu->dstAddress = srcAddr;
 InsertUsertitle(usertitle, &(*rpdu->usertitle));
 InsertConfid(confid, &(*rpdu->confid));
 return(0);
}
/******************************************************************/

