/******************************************************************/
/* filename: mc_st.c  */
/* version : 1.00 */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
#include "processConfig.h"
#include "debug.h"

/******************************************************************/
/* External declaration of pipes */
extern int MSAP_datareq_pipe[2];

/******************************************************************/
/* External declaration of functions */
extern int USAP_datareq_proc(const int usap, 
  const struct sockaddr_in *dstAddr, const char *pdu, const int maxpdusize);

/******************************************************************/
/* function prototying (forward declaration) */
int mc_st(struct ConfPartnersStruc *p);

int mc_st( struct ConfPartnersStruc *p)
{  
 int i,j;
 struct datareq_mdu_Type mdu;
 fd_set read_mask;
 Debug(Mc_st,"mc_st.c: Entering Multicast Send Task Loop.");

 while (1)
 { 
  FD_ZERO(&read_mask);
  FD_SET(MSAP_datareq_pipe[0], &read_mask);

  select(FD_SETSIZE, &read_mask, (fd_set *) 0, (fd_set *) 0, (struct timeval *) 0);

  if (FD_ISSET(MSAP_datareq_pipe[0], &read_mask))
  { /* Read a mdu from MSAP_datareq_pipe[0] */
   int read_result;
   read_result = read(MSAP_datareq_pipe[0],(char *)&mdu, \
	  sizeof(struct datareq_mdu_Type));
   if (read_result<0) 
   { 
    Error(Mc_st,"mc_st.c: Reading from pipe 'MSAP_datareq_pipe[0]'");
    return(-1); 
   }  
   else if (read_result == 0) 
   {
    return(0); /* EOF */
   }

   /* Send each partner a data message */
   for(i=0; mdu.dstAddress[i]>0; i++)
   { 
    /* Locate partners msap identifier */
    for (j=0; j<p->size ; j++)
 	if (p->Partner[j].msapid == mdu.dstAddress[i]) break;
 
    /* if partners msap identifier is located then send data */
    if (j<p->size)
    { /* Retrieve partner socketaddress and send data */
      switch (mdu.data[0])
      {
       case '\1':
       { 
        Debug(Mc_st,"mc_st.c: Sending JOIN-pdu (%d bytes) to user %d", 
 	     CalcPDUSize((char *)&mdu.data),p->Partner[j].msapid); 
 	    break;
 	   }       
 	   case '\2': 
       { 
        Debug(Mc_st,"mc_st.c: Sending LEAVE-pdu (%d bytes) to user %d", 
 	     CalcPDUSize((char *)&mdu.data),p->Partner[j].msapid); 
 	    break;
 	   }       
       case '\3': 
       { 
        Debug(Mc_st,"mc_st.c: Sending ANSWER-pdu (%d bytes) to user %d", 
 	     CalcPDUSize((char *)&mdu.data),p->Partner[j].msapid); 
 	    break;
 	   }       
       case '\4': 
       { 
        Debug(Mc_st,"mc_st.c: Sending DATA-pdu (%d bytes) to user %d", 
 	     CalcPDUSize((char *)&mdu.data),p->Partner[j].msapid); 
 	    break;
 	   }       
       default  :
       { 
        Debug(Mc_st,"mc_st.c: Sending Unknown-pdu (%d bytes) to user %d", 
 	     CalcPDUSize((char *)&mdu.data),p->Partner[j].msapid); 
 	    break;
 	   }       
      } 
 		
     if (USAP_datareq_proc(p->user.usap, \
 		  &p->Partner[j].socketaddress, \
 		  (char *)&mdu.data, \
/* 		  MAXPDUSIZE \ */
          CalcPDUSize((char *)&mdu.data) \
 		  ) !=0 \
 	   )
 	 Debug(Mc_st,"mc_st.c: Occured during performing 'USAP_datareq_proc"); 
    }
    else
 	 Debug(Mc_st,"mc_st.c: Conference partner msap id %d not found!", mdu.dstAddress[i]);
   }
  }
 }
} 

int
CalcPDUSize(const char *pdu)
{
 switch (pdu[0])
 {
  case '\1':
  case '\2':
  case '\3': return 21; 
  case '\4': return (2+pdu[1]); 
  default  : return -1; 
 } 
}

